@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010 -- leonerd@leonerd.org.uk
 
 package IO::Async::Protocol::Stream;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.60_004';
+our $VERSION = '0.30';
 
 use base qw( IO::Async::Protocol );
 
@@ -32,15 +32,16 @@ protocol.
  sub on_read
  {
     my $self = shift;
-    my ( $buffref, $eof ) = @_;
+    my ( $buffref, $closed ) = @_;
 
     return 0 unless $$buffref =~ s/^(.*)\n//;
     my $line = $1;
 
     if( $line =~ m/^HELLO (.*)/ ) {
-       my $name = $1;
+       my ( $name ) = @_;
 
-       $self->invoke_event( on_hello => $name );
+       my $on_hello = $self->{on_hello} || $self->can( 'on_hello' );
+       $on_hello->( $self, $name );
     }
 
     return 1;
@@ -59,7 +60,7 @@ protocol implementation would be likely to contain.
 
 =head1 DESCRIPTION
 
-This subclass of L<IO::Async::Protocol> is intended to stand as a base class
+This subclass of L<IO::Async:Notifier> is intended to stand as a base class
 for implementing stream-based protocols. It provides an interface similar to
 L<IO::Async::Stream>, primarily, a C<write> method and an C<on_read> event
 handler.
@@ -70,31 +71,32 @@ independence from the actual stream being used. For example, the stream may
 actually be an L<IO::Async::SSLStream> to allow the protocol to be used over
 SSL.
 
-As with C<IO::Async::Stream>, it is required that by the time the protocol
-object is added to a Loop, that it either has an C<on_read> method, or has
-been configured with an C<on_read> callback handler.
+The object may be used in one of two ways; as an instance with CODE references
+as callbacks, or as a base class with overridden methods.
 
-=cut
+=over 4
 
-=head1 EVENTS
+=item Subclassing
 
-The following events are invoked, either using subclass methods or CODE
-references in parameters:
+If a subclass is built, then it can override the following methods to handle
+events:
 
-=head2 $ret = on_read \$buffer, $eof
+ $ret = $self->on_read( \$buffer, $handleclosed )
 
-=head2 on_read_eof
+ $self->on_closed()
 
-=head2 on_write_eof
-
-The event handlers are invoked identically to C<IO::Async::Stream>.
-
-=head2 on_closed
+The C<on_read> handler is invoked identically to C<IO::Async::Stream>.
 
 The C<on_closed> handler is optional, but if provided, will be invoked after
 the stream is closed by either side (either because the C<close()> method has
 been invoked on it, or on an incoming EOF).
 
+=back
+
+As with C<IO::Async::Stream>, it is required that by the time the protocol
+object is added to a Loop, that it either has an C<on_read> method, or has
+been configured with an C<on_read> callback handler.
+
 =cut
 
 =head1 PARAMETERS
@@ -105,18 +107,15 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 =item on_read => CODE
 
-=item on_read_eof => CODE
+A CODE reference for when more data is available
 
-=item on_write_eof => CODE
+ $ret = $on_read->( $self, \$buffer, $handleclosed )
 
-CODE references for the events.
+=item on_closed => CODE
 
-=item handle => IO
+Optional. A CODE reference to invoke when the transport handle becomes closed.
 
-A shortcut for the common case where the transport only needs to be a plain
-C<IO::Async::Stream> object. If this argument is provided without a
-C<transport> object, a new C<IO::Async::Stream> object will be built around
-the given IO handle, and used as the transport.
+ $on_closed->( $self )
 
 =back
 
@@ -127,19 +126,14 @@ sub configure
    my $self = shift;
    my %params = @_;
 
-   for (qw( on_read on_read_eof on_write_eof )) {
+   for (qw( on_read on_closed )) {
       $self->{$_} = delete $params{$_} if exists $params{$_};
    }
 
-   if( !exists $params{transport} and my $handle = delete $params{handle} ) {
-      require IO::Async::Stream;
-      $params{transport} = IO::Async::Stream->new( handle => $handle );
-   }
-
    $self->SUPER::configure( %params );
 
-   if( $self->loop ) {
-      $self->can_event( "on_read" ) or
+   if( $self->get_loop ) {
+      $self->{on_read} or $self->can( "on_read" ) or
          croak 'Expected either an on_read callback or to be able to ->on_read';
    }
 }
@@ -148,7 +142,7 @@ sub _add_to_loop
 {
    my $self = shift;
 
-   $self->can_event( "on_read" ) or
+   $self->{on_read} or $self->can( "on_read" ) or
       croak 'Expected either an on_read callback or to be able to ->on_read';
 }
 
@@ -157,20 +151,25 @@ sub setup_transport
    my $self = shift;
    my ( $transport ) = @_;
 
-   $self->SUPER::setup_transport( $transport );
-
    $transport->configure( 
-      on_read => $self->_replace_weakself( sub {
-         my $self = shift or return;
-         $self->invoke_event( on_read => @_ );
+      on_read => $self->_capture_weakself( sub {
+         my $self = shift;
+         my ( $transport, $buffref, $closed ) = @_;
+
+         my $on_read = $self->{on_read} ||
+                        $self->can( 'on_read' );
+
+         $on_read->( $self, $buffref, $closed );
       } ),
-      on_read_eof => $self->_replace_weakself( sub {
-         my $self = shift or return;
-         $self->maybe_invoke_event( on_read_eof => @_ );
-      } ),
-      on_write_eof => $self->_replace_weakself( sub {
-         my $self = shift or return;
-         $self->maybe_invoke_event( on_write_eof => @_ );
+
+      on_closed => $self->_capture_weakself( sub {
+         my $self = shift;
+         my ( $transport ) = @_;
+
+         my $on_closed = $self->{on_closed} ||
+                          $self->can( 'on_closed' );
+
+         $on_closed->( $self ) if $on_closed;
       } ),
    );
 }
@@ -183,15 +182,13 @@ sub teardown_transport
    $transport->configure(
       on_read => undef,
    );
-
-   $self->SUPER::teardown_transport( $transport );
 }
 
 =head1 METHODS
 
 =cut
 
-=head2 $protocol->write( $data )
+=head2 $self->write( $data )
 
 Writes the given data by calling the C<write> method on the contained
 transport stream.
@@ -201,41 +198,16 @@ transport stream.
 sub write
 {
    my $self = shift;
-   my ( $data, %args ) = @_;
+   my ( $data ) = @_;
 
-   if( ref $data eq "CODE" ) {
-      $data = $self->_replace_weakself( $data );
-   }
-
-   if( $args{on_flush} ) {
-      $args{on_flush} = $self->_replace_weakself( $args{on_flush} );
-   }
-
-   my $transport = $self->transport or croak "Attempted to ->write to a ".ref($self)." with no transport";
-   $transport->write( $data, %args );
+   $self->transport->write( $data );
 }
 
-=head2 $protocol->connect( %args )
+# Keep perl happy; keep Britain tidy
+1;
 
-Sets up a connection to a peer, and configures the underlying C<transport> for
-the Protocol. Calls C<IO::Async::Protocol> C<connect> with C<socktype> set to
-C<"stream">.
-
-=cut
-
-sub connect
-{
-   my $self = shift;
-   $self->SUPER::connect(
-      @_,
-      socktype => "stream",
-   );
-}
+__END__
 
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
-
-=cut
-
-0x55AA;
